
#include "BitmapDisplay.h"

#include <Bitmap.h>
#include <Message.h>
#include <Handler.h>


IBitmapDisplay::IBitmapDisplay( BRect frame, const char * name )
		: BView( frame, name, B_FOLLOW_H_CENTER | B_FOLLOW_V_CENTER, B_WILL_DRAW )
{

}

IBitmapDisplay::IBitmapDisplay( BMessage *archive )
	: BView( archive )
{

}

IBitmapDisplay::~IBitmapDisplay()
{
	RemoveSelf();
}

void
IBitmapDisplay::AttachedToWindow(void)
{
	SetViewColor(255, 255, 255, 255);
}

void
IBitmapDisplay::Draw( BRect updateRect )
{
	PushState();

	SetHighColor(0, 0, 0, 255);
	StrokeRect(Bounds());
		
	SetDrawingMode(B_OP_ALPHA);
	if(mLargeBitmap)
		DrawBitmap(mLargeBitmap, Bounds().InsetBySelf(2, 2));
	PopState();
}

void
IBitmapDisplay::MouseDown( BPoint point )
{
	BMessage * msg = new BMessage(B_SIMPLE_DATA);
	
	BMessage * largeMessage = new BMessage();
	mLargeBitmap->Archive(largeMessage, true);
	BMessage * miniMessage = new BMessage();
	mMiniBitmap->Archive(miniMessage, true);
	
	msg->AddMessage("icon/large", largeMessage);
	msg->AddMessage("icon/mini", miniMessage);

	BBitmap *copy = new BBitmap(mLargeBitmap->Bounds(),
		mLargeBitmap->ColorSpace());
	copy->SetBits(mLargeBitmap->Bits(), 
		mLargeBitmap->BitsLength(), 0, 
		mLargeBitmap->ColorSpace());
	DragMessage(msg, copy, B_OP_BLEND, point);
}

void
IBitmapDisplay::SetBitmap(BBitmap * bitmap, icon_size which)
{
	if(which == B_LARGE_ICON)
		mLargeBitmap = bitmap;
	else if(which == B_MINI_ICON)
		mMiniBitmap = bitmap;
}

void
IBitmapDisplay::GetBitmap(BBitmap ** bitmap, icon_size which)
{
	if(which == B_LARGE_ICON)
		*bitmap = mLargeBitmap;
	else if(which == B_MINI_ICON)
		*bitmap = mMiniBitmap;
}
