
#include "BitmapWell.h"

#include <View.h>
#include <stdio.h>
#include <Bitmap.h>
#include <NodeInfo.h>
#include <Node.h>
#include <Archivable.h>
#include <Handler.h>
#include <Window.h>


//
// Default Constructor
//

IBitmapWell::IBitmapWell( BRect frame, const char *name, BMessage * send_message)
		: BView( frame, name, B_FOLLOW_V_CENTER, B_WILL_DRAW/* | B_NAVIGABLE */)
{
	mMessage = send_message;
	mLargeBitmap = new BBitmap(BRect(0, 0, 31, 31), B_CMAP8);
	mMiniBitmap = new BBitmap(BRect(0, 0, 15, 15), B_CMAP8);
	
	mHasStuff = false;
}


//
// Destructor
//

IBitmapWell::~IBitmapWell()
{
	RemoveSelf();
	delete mLargeBitmap;
	delete mMiniBitmap;
	delete mMessage;
}

void
IBitmapWell::AttachedToWindow(void)
{
	SetViewColor( 255, 255, 255 );
	ResizeTo(mLargeBitmap->Bounds().Width() + 4, mLargeBitmap->Bounds().Height() + 4);
}

void
IBitmapWell::Draw( BRect updateRect )
{
	PushState();
	
	SetHighColor(0, 0, 0, 255);
	StrokeRect(Bounds());

	SetDrawingMode(B_OP_ALPHA);
	DrawBitmap(mLargeBitmap, BPoint(Bounds().left + 2, Bounds().top + 2) );
	
	PopState();
}


/*
void
IBitmapWell::KeyDown( const char *bytes, int32 numBytes )
{

}
*/

void
IBitmapWell::MessageReceived( BMessage * message )
{
	switch( message->what )
	{
		case B_SIMPLE_DATA:
		{
			entry_ref ref;
			if(message->FindRef("refs", &ref) == B_OK)
			{			
				BNodeInfo * nodeInfo = new BNodeInfo( new BNode(&ref) );
				BBitmap * small_bitmap = new BBitmap(BRect(0, 0, 15, 15), B_CMAP8);
				BBitmap * large_bitmap = new BBitmap(BRect(0, 0, 31, 31), B_CMAP8);
				nodeInfo->GetTrackerIcon(small_bitmap, B_MINI_ICON);
				nodeInfo->GetTrackerIcon(large_bitmap, B_LARGE_ICON);
				SetBitmap(small_bitmap, B_MINI_ICON);
				SetBitmap(large_bitmap, B_LARGE_ICON);
			
				Window()->PostMessage(mMessage);// tell em it changed
				
				Invalidate();
				mHasStuff = true;
				break;
			}
			
			//printf("B_SIMPLE_DATA, but not the Tracker's...\n");
			//message->PrintToStream();		
		}
		default:
		{
			if(message->HasMessage("icon/large") && message->HasMessage("icon/mini"))
			{	// It's got archived bitmaps!
				BMessage * message1 = new BMessage(),
						 * message2 = new BMessage();
				message->FindMessage("icon/large", message1);
				message->FindMessage("icon/mini", message2);
				BArchivable * largeArchive = instantiate_object(message1);
				BArchivable * miniArchive = instantiate_object(message2);
				if(largeArchive && miniArchive)
				{
					BBitmap * largeBitmap = dynamic_cast<BBitmap *>(largeArchive);
					BBitmap * miniBitmap = dynamic_cast<BBitmap *>(miniArchive);

					if(largeBitmap && miniBitmap)
					{
						SetBitmap(largeBitmap, B_LARGE_ICON);			
						SetBitmap(miniBitmap, B_MINI_ICON);
		
						Window()->PostMessage(mMessage);// tell em it changed
					
						Invalidate();
						mHasStuff = true;
					}
				}
				
			}
			else
			{
				//printf("(what's this?)\n");
				//message->PrintToStream();
				BView::MessageReceived(message);
			}
			break;
		}
	}
}



//
// MouseDown
//


void
IBitmapWell::MouseDown( BPoint point )
{
	BMessage * msg = new BMessage(B_SIMPLE_DATA);
	
	BMessage * largeMessage = new BMessage();
	mLargeBitmap->Archive(largeMessage, true);
	BMessage * miniMessage = new BMessage();
	mMiniBitmap->Archive(miniMessage, true);
	
	msg->AddMessage("icon/large", largeMessage);
	msg->AddMessage("icon/mini", miniMessage);

	BBitmap *copy = new BBitmap(mLargeBitmap->Bounds(),
		mLargeBitmap->ColorSpace());
	copy->SetBits(mLargeBitmap->Bits(), 
		mLargeBitmap->BitsLength(), 0, 
		mLargeBitmap->ColorSpace());
	DragMessage(msg, copy, B_OP_BLEND, point);
}




//
// MouseMoved
//

/*
void
IBitmapWell::MouseMoved( BPoint point, uint32 transit, const BMessage *message )
{

}
*/



//
// MouseUp
//

/*
void
IBitmapWell::MouseUp( BPoint point )
{

}
*/


void
IBitmapWell::SetBitmap(BBitmap * bitmap, icon_size which)
{
	if(which == B_LARGE_ICON)
		mLargeBitmap = bitmap;
	else if(which == B_MINI_ICON)
		mMiniBitmap = bitmap;
}

status_t
IBitmapWell::GetBitmap(BBitmap ** bitmap, icon_size which)
{
	if(which == B_LARGE_ICON)
	{
		*bitmap = mLargeBitmap;
	}
	else if(which == B_MINI_ICON)
	{
		*bitmap = mMiniBitmap;
	}
	else
	{
		return B_ERROR;
	}

	return B_OK;
}

bool
IBitmapWell::HasStuff(void)
{
	return mHasStuff;
}
