#include "View.h"

#include <View.h>
#include <Button.h>
#include <Bitmap.h>
#include <Directory.h>
#include <NodeInfo.h>
#include <Node.h>
#include <Application.h>
#include <Box.h>
#include <FindDirectory.h>
#include <StringView.h>
#include <Handler.h>
#include <Window.h>

#include <stdio.h>
#include <stdlib.h>



//
// Default Constructor
//

IView::IView( BRect frame, const char *name,
				uint32 resizingMode, uint32 flags )
		: BView( frame, name, resizingMode, flags )
{
	
	find_directory(B_USER_SETTINGS_DIRECTORY, &mSettingsPath);
	mSettingsPath.Append("Iconblender_Settings");
	
	//mIconMode = T_MINI_ICON_MODE;

	// Blendee well and default folder icon
	BRect rect(15, 30, 15 + 34, 30 + 34);
	mBlendeeWell = new IBitmapWell(rect, "blendee", new BMessage(T_BLENDEE_CHANGED));		
	
	BRect stringRect(mBlendeeWell->Frame().left - 10, mBlendeeWell->Frame().top - be_plain_font->Size() - 2, mBlendeeWell->Frame().right + 12, mBlendeeWell->Frame().top - 1);
	BStringView * tBlendeeString = new BStringView(stringRect, "blendee_string", "Background", mBlendeeWell->ResizingMode());
//	tBlendeeString->ResizeToPreferred();
	
	AddChild(tBlendeeString);
	AddChild(mBlendeeWell);


	// Blender well (blank)	
	rect.OffsetBy(0, 40 + (be_plain_font->Size() * 2));
	mBlenderWell = new IBitmapWell(rect, "blender", new BMessage(T_BLENDER_CHANGED));

	stringRect.Set(mBlenderWell->Frame().left - 10, mBlenderWell->Frame().top - be_plain_font->Size() - 1, mBlenderWell->Frame().right + 12, mBlenderWell->Frame().top - 1);
	BStringView * tBlenderString = new BStringView(stringRect, "blender_string", "Foreground", mBlenderWell->ResizingMode());
//	tBlenderString->ResizeToPreferred();
	
	AddChild(tBlenderString);
	AddChild(mBlenderWell);
		

	// 'Blend' Box	
	rect.Set(mBlendeeWell->Frame().right + 25, Bounds().top + 4, Bounds().right - 4, Bounds().bottom - 4);
	BBox * boxen = new BBox(rect, "box", B_FOLLOW_ALL);
	boxen->SetLabel("Blend");
	AddChild(boxen);
	
	// Displays
	rect.Set(boxen->Bounds().right - 76, boxen->Bounds().top + 16, boxen->Bounds().right - 8, boxen->Bounds().top + 82 + 2);	
	mDisplay =  new IBitmapDisplay(rect, "mDisplay");

	rect.Set(mDisplay->Frame().left, mDisplay->Frame().bottom + 2, mDisplay->Frame().left + 32 + 3, mDisplay->Frame().bottom + 2 + 32 + 3);	
	mMidDisplay =  new IBitmapDisplay(rect, "mDisplay");

	rect.Set(mMidDisplay->Frame().right + 4, mDisplay->Frame().bottom + 2, mMidDisplay->Frame().right + 4 + 16 + 3, mDisplay->Frame().bottom + 2 + 16 + 3);	
	mMiniDisplay =  new IBitmapDisplay(rect, "mDisplay");

	//printf("disp: %f %f  mid: %f %f  mini: %f %f\n", mDisplay->Bounds().Width(), mDisplay->Bounds().Height(), mMidDisplay->Bounds().Width(), mMidDisplay->Bounds().Height(), mMiniDisplay->Bounds().Width(), mMiniDisplay->Bounds().Height());
	boxen->AddChild(mDisplay);
	boxen->AddChild(mMidDisplay);
	boxen->AddChild(mMiniDisplay);

	// "Large icon" and "Mini icon" radio buttons.
	rect.Set(boxen->Bounds().left + 8, boxen->Frame().top + 20, boxen->Bounds().left + 75, boxen->Frame().top + 30);
	mMiniRadio = new BRadioButton(rect, "mini_icon_radio", "Mini Icon", new BMessage(T_ICON_MODE_CHANGED), B_FOLLOW_H_CENTER | B_FOLLOW_V_CENTER);
	mMiniRadio->ResizeToPreferred();
	mMiniRadio->SetValue(true);
	
	rect.OffsetBy(0, 20);
	mLargeRadio = new BRadioButton(rect, "large_icon_radio", "Large Icon", new BMessage(T_ICON_MODE_CHANGED), B_FOLLOW_H_CENTER | B_FOLLOW_V_CENTER);
	mLargeRadio->ResizeToPreferred();

	boxen->AddChild(mMiniRadio);
	boxen->AddChild(mLargeRadio);
	
	// Coord controls
	rect.Set(boxen->Bounds().left + 25, mLargeRadio->Frame().bottom + 3, boxen->Bounds().left + 75, mLargeRadio->Frame().bottom + 20);
	mXControl = new BTextControl(rect, "x_control", "X:", "", new BMessage(T_X_CHANGED), B_FOLLOW_H_CENTER | B_FOLLOW_V_CENTER);
	mXControl->SetDivider(10);
	
	rect.OffsetBy(0, 20);

	mYControl = new BTextControl(rect, "y_control", "Y:", "", new BMessage(T_X_CHANGED), B_FOLLOW_H_CENTER | B_FOLLOW_V_CENTER);
	mYControl->SetDivider(10);
	
	boxen->AddChild(mXControl);
	boxen->AddChild(mYControl);


	
}

IView::~IView()
{
	SavePrefs();

	delete mBlendeeWell;
	delete mBlenderWell;
	delete mDisplay;
	delete mMidDisplay;
	delete mMiniDisplay;

}

void
IView::AttachedToWindow(void)
{
	SetViewColor( ui_color( B_PANEL_BACKGROUND_COLOR ) );

	LoadPrefs();

	Blend();

}

/*
void
IView::Draw( BRect updateRect )
{

}
*/

void
IView::MessageReceived( BMessage *message )
{
	switch( message->what )
	{
		case T_BLEND:
		{
			break;
		}
		
		case T_BLENDEE_CHANGED:
		case T_BLENDER_CHANGED:
		{
			Blend();
			break;
		}
		
		case T_X_CHANGED:
		case T_Y_CHANGED:
		{
			int32 x = atoi(mXControl->Text());
			int32 y = atoi(mYControl->Text());
			SetPosition(x, y);
			Blend();
			break;
		
		}
		case T_ICON_MODE_CHANGED:
		{
			// Lame hack.
			if(mLargeRadio->Value() == 1)
			{
				SetPosition(T_LARGE_X_DEFAULT, T_LARGE_Y_DEFAULT);
				mIconMode = T_LARGE_ICON_MODE;
			}
			else if(mMiniRadio->Value() == 1)
			{
				SetPosition(T_MINI_X_DEFAULT, T_MINI_Y_DEFAULT);
				mIconMode = T_MINI_ICON_MODE;
			}
			
			Blend();
			
			break;
		}
		default:
		{
			BView::MessageReceived(message);
			break;
		}
	}
}

void
IView::MouseDown(BPoint point)
{
	Window()->Activate();
}

status_t
IView::Blend()
{
	printf("Blending...\n");
	BBitmap * blendee = new BBitmap(BRect(0, 0, 31, 31), B_CMAP8);
	BBitmap * blender;
	BBitmap * bigBuffer = new BBitmap(BRect(0, 0, 31, 31), B_CMAP8, true);
	BBitmap * littleBuffer = new BBitmap(BRect(0, 0, 15, 15), B_CMAP8, true);

	mBlendeeWell->GetBitmap(&blendee, B_LARGE_ICON);

	if(mIconMode == T_MINI_ICON_MODE)
	{
		blender = new BBitmap(BRect(0, 0, 15, 15), B_CMAP8);
		if(mBlenderWell->GetBitmap(&blender, B_MINI_ICON) != B_OK)
			printf("ERROR: Couldn't get blendee icon.\n");
	}
	else if(mIconMode == T_LARGE_ICON_MODE)
	{
		blender = new BBitmap(BRect(0, 0, 31, 31), B_CMAP8);
		if(mBlenderWell->GetBitmap(&blender, B_LARGE_ICON) != B_OK)
			printf("ERROR: Couldn't get blender icon!\n");
	}

	// Blend the big'un.
	BView * blendView = new BView(bigBuffer->Bounds(), "blend-o-rama!", 0, B_WILL_DRAW);
	bigBuffer->AddChild(blendView);
	
	if(blendView->LockLooper())
	{			
		blendView->DrawBitmapAsync(blendee, blendView->Bounds());
		blendView->SetDrawingMode(B_OP_ALPHA);
		if(mBlenderWell->HasStuff())
			blendView->DrawBitmapAsync(blender, mPosition);
		blendView->Sync();
	
		blendView->UnlockLooper();
		bigBuffer->RemoveChild(blendView);
	}
	
	//Blend the itty bitty'un.
	mBlendeeWell->GetBitmap(&blendee, B_MINI_ICON);
	
	blendView = new BView(littleBuffer->Bounds(), "blend-o-rama!", 0, B_WILL_DRAW);
	littleBuffer->AddChild(blendView);
	if(blendView->LockLooper())
	{
		blendView->SetHighColor(255, 0, 0, 100);
					
		blendView->DrawBitmapAsync(blendee, blendView->Bounds());
		blendView->SetDrawingMode(B_OP_ALPHA);
		BRect blenderRect(mPosition.x / 2, mPosition.y / 2, (mPosition.x / 2) + (blender->Bounds().Width() / 2), (mPosition.y / 2) + (blender->Bounds().Height() / 2));
		if(mBlenderWell->HasStuff())
			blendView->DrawBitmapAsync(blender, blenderRect);
		blendView->Sync();
	
		blendView->UnlockLooper();
		littleBuffer->RemoveChild(blendView);
	}
	
	// Put them in the displays!	
	mDisplay->SetBitmap(bigBuffer, B_LARGE_ICON);
	mDisplay->SetBitmap(littleBuffer, B_MINI_ICON);

	mMidDisplay->SetBitmap(bigBuffer, B_LARGE_ICON);
	mMidDisplay->SetBitmap(littleBuffer, B_MINI_ICON);

	mMiniDisplay->SetBitmap(littleBuffer, B_LARGE_ICON);
	mMiniDisplay->SetBitmap(littleBuffer, B_MINI_ICON);

	mDisplay->Invalidate();
	mMidDisplay->Invalidate();
	mMiniDisplay->Invalidate();
	
	return B_OK;
}

void
IView::SetPosition(int x, int y)
{
	mPosition.Set(x, y);

	char	* xString = new char,
			* yString = new char;

	sprintf(xString, "%i", x);
	sprintf(yString, "%i", y);

	
	mXControl->SetText(xString);
	mYControl->SetText(yString);
}

void
IView::SavePrefs(void)
{
	printf("Saving prefs...\n");
	BMessage * prefs = new BMessage();

	// Dump bitmaps.
	BMessage * largeMessage = new BMessage();
	BMessage * miniMessage = new BMessage();
	BBitmap * largeBitmap;
	BBitmap * miniBitmap;

	mBlendeeWell->GetBitmap(&largeBitmap, B_LARGE_ICON);
	mBlendeeWell->GetBitmap(&miniBitmap, B_MINI_ICON);
	
	largeBitmap->Archive(largeMessage);
	miniBitmap->Archive(miniMessage);
	
	prefs->AddMessage("jw:large_bitmap", largeMessage);
	prefs->AddMessage("jw:mini_bitmap", miniMessage);
	
	// Dump the other stuff.
	prefs->AddInt8("jw:icon_mode", mIconMode);
	prefs->AddPoint("jw:position", mPosition);
		
	// Write it to disk
	BFile * file = new BFile( mSettingsPath.Path(), B_WRITE_ONLY | B_ERASE_FILE | B_CREATE_FILE );
	if( prefs->Flatten(file) != B_OK )
		printf("ERROR: Couldn't flatten prefs file.!\n");

	printf("Done saving.\n");	
}

void
IView::LoadPrefs(void)
{
	printf("Loading prefs...\n");
	BMessage * prefs = new BMessage();
	BFile * file = new BFile( mSettingsPath.Path(), B_READ_ONLY );
	if(prefs->Unflatten( file ) != B_OK)
	{
		LoadDefaults();
		return;
	}
		

	if(	/*prefs->HasMessage("jw:mini_bitmap") &&*/ prefs->HasMessage("jw:large_bitmap") &&
		prefs->HasInt8("jw:icon_mode") && prefs->HasPoint("jw:position") )
	{
		mDefaultRun = false;

		BMessage 	* miniMessage = new BMessage();
		BMessage	* largeMessage = new BMessage();

		prefs->FindMessage("jw:large_bitmap", largeMessage);
		prefs->FindMessage("jw:mini_bitmap", miniMessage);
	
		BArchivable * largeArchive = instantiate_object(largeMessage);
		BArchivable * miniArchive = instantiate_object(miniMessage);
	
		if(largeArchive && miniArchive)
		{
			BBitmap * largeBitmap = dynamic_cast<BBitmap *>(largeArchive);
			BBitmap * miniBitmap = dynamic_cast<BBitmap *>(miniArchive);

			if(largeBitmap && miniBitmap)
			{

				mBlendeeWell->SetBitmap(largeBitmap, B_LARGE_ICON);
				mBlendeeWell->SetBitmap(miniBitmap, B_MINI_ICON);
			}
			else
			{
				printf("ERROR: Couldn't get largeBitmap or miniBitmap from prefs.\n");
				return;
			}
		}	

		prefs->FindInt8("jw:icon_mode", &mIconMode);
		if(mIconMode == T_MINI_ICON_MODE)
			mMiniRadio->SetValue(1);
		else if(mIconMode == T_LARGE_ICON_MODE)
			mLargeRadio->SetValue(1);
			
		prefs->FindPoint("jw:position", &mPosition);
		SetPosition(mPosition.x, mPosition.y);		
	
	}
	else
	{
		LoadDefaults();
		return;
	}

	printf("Done loading.\n");
}

void
IView::LoadDefaults(void)
{
	printf("Loading default prefs...\n");
	mDefaultRun = true;
	mIconMode = T_MINI_ICON_MODE;
	SetPosition(T_MINI_X_DEFAULT, T_MINI_Y_DEFAULT);
	
	BMimeType * folderMime = new BMimeType("application/x-vnd.Be-directory");
	BBitmap * folderLargeBitmap = new BBitmap(BRect(0, 0, 31, 31), B_CMAP8);
	BBitmap * folderMiniBitmap = new BBitmap(BRect(0, 0, 15, 15), B_CMAP8);

	folderMime->GetIcon(folderLargeBitmap, B_LARGE_ICON);
	folderMime->GetIcon(folderMiniBitmap, B_MINI_ICON);	

	mBlendeeWell->SetBitmap(folderLargeBitmap, B_LARGE_ICON);
	mBlendeeWell->SetBitmap(folderMiniBitmap, B_MINI_ICON);
	
	printf("Done loading defaults.\n");
}
